// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.intellij.keymap;

import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;

import java.util.*;

public abstract class KeymapsTestCase extends KeymapsTestCaseBase {
  @NonNls
  protected static final Map<String, String[][]> DEFAULT_DUPLICATES = new HashMap<>(){{
    put("$default", new String[][] {
    { "ADD",                      "ExpandTreeNode", "Graph.ZoomIn"},
    { "DELETE",                   "$Delete", "DatabaseView.DropAction"},
    { "ENTER",                    "Console.Execute", "Console.TableResult.EditValue", "DirDiffMenu.SynchronizeDiff", "EditorChooseLookupItem",
                                  "EditorEnter", "NextTemplateVariable",
                                  "Git.Log.Branches.Change.Branch.Filter", "SplitChooser.Split"},
    { "F1",                       "ContextHelp", "org.jetbrains.r.actions.REditorHelpAction"},
    { "F2",                       "GotoNextError", "GuiDesigner.EditComponent", "GuiDesigner.EditGroup", "Console.TableResult.EditValue", "XDebugger.SetValue", "Git.Reword.Commit", "Git.Rename.Local.Branch", "ShelvedChanges.Rename", "ChangesView.Rename"},
    { "alt ENTER",                "ShowIntentionActions", "Console.TableResult.EditValue"},
    { "F3",                       "FindNext", "PoToggleSelection"},
    { "F5",                       "Graph.RouteEdges", "CopyElement"},
    { "F7",                       "NextDiff", "StepInto"},
    { "INSERT",                   "EditorToggleInsertState", "UsageView.Include", "DomElementsTreeView.AddElement", "DomCollectionControl.Add", "XDebugger.NewWatch"},
    { "SUBTRACT",                 "CollapseTreeNode", "Graph.ZoomOut"},
    { "TAB",                      "EditorChooseLookupItemReplace", "NextTemplateVariable", "NextParameter", "EditorIndentSelection",
                                  "EditorTab", "NextTemplateParameter", "ExpandLiveTemplateByTab", "BraceOrQuoteOut", "SplitChooser.NextWindow"},
    { "alt DOWN",                 "ShowContent", "MethodDown", "ShowSearchHistory"},
    { "alt F1",                   "SelectIn", "ProjectViewChangeView"},
    { "alt F6",                   "CommanderSyncViews", "EditPropertyValue"},
    { "alt INSERT",               "FileChooser.NewFolder", "Generate", "NewElement"},
    { "alt LEFT",                 "Diff.PrevChange", "PreviousTab"},
    { "alt RIGHT",                "Diff.NextChange", "NextTab"},
    { "control 1",                "FileChooser.GotoHome", "GotoBookmark1", "DuplicatesForm.SendToLeft"},
    { "control 2",                "FileChooser.GotoProject", "GotoBookmark2", "DuplicatesForm.SendToRight"},
    { "control 3",                "GotoBookmark3", "FileChooser.GotoModule"},
    { "control ADD",              "ExpandAll", "ExpandExpandableComponent", "ExpandRegion"},
    { "control DIVIDE",           "CommentByLineComment", "Graph.ActualSize"},
    { "control DOWN",             "EditorScrollDown", "EditorLookupDown", "MethodOverloadSwitchDown", "JupyterSelectCellBelowAction"},
    { "control ENTER",            "Console.Execute.Multiline", "DirDiffMenu.SynchronizeDiff.All", "EditorSplitLine", "NotebookRunCellAction", "PyExecuteCellAction", "SplitChooser.Duplicate", "Terminal.SmartCommandExecution.Run", "ViewSource", "org.jetbrains.r.actions.RunSelection"},
    { "control TAB",              "Switcher", "Diff.FocusOppositePane"},
    { "control EQUALS",           "ExpandAll", "ExpandExpandableComponent", "ExpandRegion"},
    { "control F5",               "Refresh", "Rerun"},
    { "control C",                "$Copy", "Terminal.CopySelectedText", "org.jetbrains.r.console.RConsoleView.RInterruptAction"},
    { "control D",                "EditorDuplicate", "Diff.ShowDiff", "CompareTwoFiles", "SendEOF", "FileChooser.GotoDesktop", "org.jetbrains.r.console.RConsoleView.REofAction"},
    { "control E",                "RecentFiles", "SwitcherIterateItems", "SwitcherRecentEditedChangedToggleCheckBox", "Vcs.ShowMessageHistory"},
    { "control INSERT",           "$Copy", "Terminal.CopySelectedText"},
    { "control M",                "EditorScrollToCenter", "Vcs.ShowMessageHistory"},
    { "control N",                "FileChooser.NewFolder", "GotoClass", "GotoChangedFile"},
    { "control P",                "FileChooser.TogglePathShowing", "ParameterInfo"},
    { "control R",                "Replace", "org.jetbrains.plugins.ruby.rails.console.ReloadSources"},
    { "control SLASH",            "CommentByLineComment", "Graph.ActualSize"},
    { "control SPACE",            "CodeCompletion", "ChangesView.SetDefault"},
    { "control U",                "GotoSuperMethod", "CommanderSwapPanels"},
    { "control UP",               "EditorScrollUp", "EditorLookupUp", "MethodOverloadSwitchUp", "JupyterSelectCellAboveAction"},
    { "control SUBTRACT",         "CollapseAll", "CollapseExpandableComponent", "CollapseRegion"},
    { "control V",                "$Paste", "Terminal.Paste"},
    { "control alt A",            "ChangesView.AddUnversioned", "Diagram.DeselectAll"},
    { "control alt B",               "GotoImplementation", "org.jetbrains.r.rendering.chunk.RunChunksAboveAction"},
    { "control alt D",            "Console.TableResult.SetDefault", "org.jetbrains.r.actions.RMarkdownInterruptAction"},
    { "control alt K",            "Git.Commit.And.Push.Executor", "Hg.Commit.And.Push.Executor"},
    { "control alt E",            "Console.History.Browse", "PerforceDirect.Edit", "ToggleFindInSelection"},
    { "control alt DOWN",         "NextOccurence", "Console.TableResult.NextPage"},
    { "control alt G",            "DatabaseView.SqlGenerator", "org.jetbrains.plugins.ruby.rails.actions.generators.GeneratorsPopupAction", "Mvc.RunTarget"},
    { "control alt I",             "AutoIndentLines", "RMarkdownNewChunk"},
    { "control alt R",            "org.jetbrains.plugins.ruby.tasks.rake.actions.RakeTasksPopupAction", "Django.RunManageTaskAction"},
    { "control alt UP",           "PreviousOccurence", "Console.TableResult.PreviousPage"},
    { "control alt N",            "Inline", "Console.TableResult.SetNull"},
    { "control alt X",            "SECOND_STROKE_SHORTCUT", "org.jetbrains.r.rendering.chunk.DebugChunkAction"},
    { "ctrl alt C",               "IntroduceConstant", "org.jetbrains.r.rendering.chunk.RunChunkAction"},
    { "ctrl alt U",               "ShowUmlDiagramPopup", "ChangesView.UnshelveSilently"},
    { "ctrl alt F",               "IntroduceField", "ShowFilterPopup"},
    { "control MINUS",            "CollapseAll", "CollapseExpandableComponent", "CollapseRegion"},
    { "control PERIOD",           "EditorChooseLookupItemDot", "CollapseSelection"},
    { "shift DELETE",             "$Cut", "Maven.Uml.Exclude"},
    { "shift ENTER",              "CollapseExpandableComponent", "Console.TableResult.EditValueMaximized", "DatabaseView.PropertiesAction", "EditorStartNewLine", "ExpandExpandableComponent", "NotebookRunCellSelectBelowAction", "OpenInRightSplit", "RConsoleNextLineAction"},
    { "shift F4",                 "Debugger.EditTypeSource", "EditSourceInNewWindow"},
    { "shift F6",                 "RenameElement", "Git.Reword.Commit", "Git.Rename.Local.Branch", "ShelvedChanges.Rename", "ChangesView.Rename"},
    { "shift F7",                 "PreviousDiff", "SmartStepInto"},
    { "shift INSERT",             "$Paste", "Terminal.Paste"},
    { "shift TAB",                "PreviousTemplateVariable", "PrevParameter", "EditorUnindentSelection", "PrevTemplateParameter", "SplitChooser.PreviousWindow"},
    { "shift alt ENTER",          "Console.TableResult.EditValueMaximized", "JupyterDebugAction", "SplitChooser"},
    { "shift alt LEFT",           "Diff.ApplyRightSide", "PreviousEditorTab"},
    { "shift alt RIGHT",          "Diff.ApplyLeftSide", "NextEditorTab"},
    { "shift alt A",              "NotebookInsertCellAboveAction", "VcsHistory.ShowAllAffected"},
    { "shift alt B",              "NotebookInsertCellBelowAction", "tasks.open.in.browser", "sql.SelectInDatabaseView"},
    { "shift alt G",              "EditorAddCaretPerSelectedLine", "hg4idea.QGotoFromPatches"},
    { "shift alt M",              "ChangesView.Move", "Vcs.MoveChangedLinesToChangelist"},
    { "shift ctrl C",             "CopyPaths", "DatabaseView.CopyDdlAction"},
    { "shift control D",          "TagDocumentationNavigation", "Diff.ShowSettingsPopup", "Uml.ShowDiff", "Console.TableResult.CompareCells"},
    { "shift control DOWN",       "ResizeToolWindowDown", "MoveStatementDown"},
    { "shift control ENTER",      "EditorCompleteStatement", "Console.Jpa.GenerateSql", "org.jetbrains.r.actions.RRunAction", "Terminal.SmartCommandExecution.Debug"},
    { "shift control F10",        "Console.Open", "RunClass", "RunTargetAction"},
    { "shift control F8",         "ViewBreakpoints", "EditBreakpoint"},
    { "shift control G",          "ClassTemplateNavigation", "GoToClass"},
    { "shift control H",          "ChangesView.ShelveSilently", "MethodHierarchy"},
    { "shift control LEFT",       "EditorPreviousWordWithSelection", "ResizeToolWindowLeft"},
    { "shift control M",          "EditorMatchBrace", "RInsertPipeAction"},
    { "shift control RIGHT",      "EditorNextWordWithSelection", "ResizeToolWindowRight"},
    { "shift control T",          "GotoTest", "ServiceView.ShowServices"},
    { "shift control UP",         "ResizeToolWindowUp", "MoveStatementUp"},
    { "shift control U",          "ShelveChanges.UnshelveWithDialog", "EditorToggleCase"},
    { "shift control alt D",      "UML.ShowChanges", "Console.TableResult.CloneColumn"},
    { "shift control alt ENTER",  "Console.Jpa.GenerateDDL", "Console.Transaction.Commit", "NotebookRunAllAction", "org.jetbrains.r.actions.RDebugAction"},
    { "shift ctrl alt P",         "IntroduceFunctionalParameter", "ReformatWithPrettierAction"},
    { "control alt Z",            "Vcs.RollbackChangedLines", "ChangesView.Revert"},
    { "shift control TAB",        "Switcher", "Diff.FocusOppositePaneAndScroll"},
    { "ctrl alt ENTER",           "EditorStartNewLineBefore", "QuickActionPopup", "org.jetbrains.r.actions.DebugSelection"},
    { "alt button1",              "EditorCreateRectangularSelectionOnMouseDrag", "QuickEvaluateExpression"},
    { "button2",                  "EditorPasteFromX11", "GotoDeclaration", "EditorCreateRectangularSelectionOnMouseDrag"},
    { "ESCAPE",                   "CloseGotItTooltip", "EditorEscape", "FocusEditor", "Terminal.SwitchFocusToEditor"},
    { "shift ctrl alt LEFT",      "MoveElementLeft", "Terminal.MoveToolWindowTabLeft"},
    { "shift ctrl alt RIGHT",     "MoveElementRight", "Terminal.MoveToolWindowTabRight"},
    { "SPACE",                    "Console.TableResult.ColumnVisibility", "QuickPreview", "SplitChooser.SplitCenter"},
    { "ctrl L",                   "FindNext", "Terminal.ClearBuffer"},
    { "control alt button1",      "GotoImplementation", "UiInspector"},
    });
    put("Mac OS X 10.5+", new String[][] {
    { "F1",                       "QuickJavaDoc", "org.jetbrains.r.actions.REditorHelpAction"},
    { "F3",                       "PoToggleSelection", "ToggleBookmark"},
    { "Force touch",              "GotoDeclaration", "RunToCursor"},
    { "BACK_SPACE",               "$Delete", "EditorBackSpace"},
    { "shift BACK_SPACE",         "EditorBackSpace", "UsageView.Include"},
    { "meta BACK_SPACE",          "EditorDeleteLine", "$Delete"},
    { "control UP",               "JupyterSelectCellAboveAction", "MethodUp"},
    { "control TAB",              "Switcher", "Diff.FocusOppositePane"},
    { "control D",                "Debug", "org.jetbrains.r.console.RConsoleView.REofAction"},
    { "control M",                "EditorMatchBrace", "Vcs.ShowMessageHistory"},
    { "control O",                "ExportToTextFile", "OverrideMethods"},
    { "ctrl alt F",               "EditorNextWord", "ShowFilterPopup"},
    { "control ENTER",            "Generate", "NotebookRunCellAction", "NewElement", "PyExecuteCellAction"},
    { "control SPACE",            "CodeCompletion", "ChangesView.SetDefault"},
    { "shift control TAB",        "Switcher", "Diff.FocusOppositePaneAndScroll"},
    { "meta R",                   "Refresh", "Rerun", "Replace", "org.jetbrains.plugins.ruby.rails.console.ReloadSources"},
    { "meta 1",                   "ActivateProjectToolWindow", "FileChooser.GotoHome", "DuplicatesForm.SendToLeft"},
    { "meta 2",                   "ActivateFavoritesToolWindow", "FileChooser.GotoProject", "DuplicatesForm.SendToRight"},
    { "meta 3",                   "ActivateFindToolWindow", "FileChooser.GotoModule"},
    { "meta N",                   "FileChooser.NewFolder", "Generate", "NewElement"},
    { "meta O",                   "GotoClass", "GotoChangedFile"},
    { "shift ctrl LEFT",          "Diff.ApplyRightSide", "PreviousEditorTab"},
    { "shift ctrl RIGHT",         "Diff.ApplyLeftSide", "NextEditorTab"},
    { "shift meta G",             "ClassTemplateNavigation", "GoToClass", "FindPrevious"},
    { "shift meta M",             "ChangesView.Move", "RInsertPipeAction", "Vcs.MoveChangedLinesToChangelist"},
    { "shift meta L",             "Jdbc.OpenConsole.New", "rspec.LetIntroduce"},
    { "shift meta LEFT",          "EditorLineStartWithSelection", "ResizeToolWindowLeft"},
    { "shift meta RIGHT",         "EditorLineEndWithSelection", "ResizeToolWindowRight"},
    { "shift meta CLOSE_BRACKET", "Diff.NextChange", "NextTab"},
    { "shift meta OPEN_BRACKET",  "Diff.PrevChange", "PreviousTab"},
    { "alt R",                    "Django.RunManageTaskAction", "org.jetbrains.plugins.ruby.tasks.rake.actions.RakeTasksPopupAction"},
    { "alt DOWN",                 "EditorUnSelectWord", "MethodOverloadSwitchDown", "ShowSearchHistory"},
    { "alt UP",                   "EditorSelectWord", "MethodOverloadSwitchUp"},
    { "meta L",                   "GotoLine", "Terminal.ClearBuffer"},
    { "meta K",                   "CheckinProject", "Terminal.ClearBuffer"},
    });
    put("Mac OS X", new String[][] {
    { "Force touch",              "GotoDeclaration", "RunToCursor"},
    { "BACK_SPACE",               "$Delete", "EditorBackSpace"},
    { "control LEFT",             "Diff.PrevChange", "PreviousTab"},
    { "control RIGHT",            "Diff.NextChange", "NextTab"},
    { "control DOWN",             "EditorLookupDown", "JupyterSelectCellBelowAction", "MethodDown"},
    { "control UP",               "EditorLookupUp", "JupyterSelectCellAboveAction", "MethodUp"},
    { "control ENTER",            "Generate", "NotebookRunCellAction", "NewElement", "PyExecuteCellAction"},
    { "control F5",               "Refresh", "Rerun"},
    { "control TAB",              "Switcher", "Diff.FocusOppositePane"},
    { "control N",                "Generate", "NewElement"},
    { "shift control TAB",        "Switcher", "Diff.FocusOppositePaneAndScroll"},
    { "meta 1",                   "ActivateProjectToolWindow", "FileChooser.GotoHome", "DuplicatesForm.SendToLeft"},
    { "meta 2",                   "ActivateFavoritesToolWindow", "FileChooser.GotoProject", "DuplicatesForm.SendToRight"},
    { "meta 3",                   "ActivateFindToolWindow", "FileChooser.GotoModule"},
    { "shift ctrl LEFT",          "Diff.ApplyRightSide", "PreviousEditorTab"},
    { "shift ctrl RIGHT",         "Diff.ApplyLeftSide", "NextEditorTab"},
    { "shift meta LEFT",          "EditorLineStartWithSelection", "ResizeToolWindowLeft"},
    { "shift meta RIGHT",         "EditorLineEndWithSelection", "ResizeToolWindowRight"},
    { "alt R",                    "Django.RunManageTaskAction", "org.jetbrains.plugins.ruby.tasks.rake.actions.RakeTasksPopupAction"},
    { "ctrl SPACE",               "ChangesView.SetDefault", "CodeCompletion"},
    { "meta K",                   "CheckinProject", "Terminal.ClearBuffer"},
    });
    put("Emacs", new String[][] {
    { "TAB",                      "EditorChooseLookupItemReplace", "NextTemplateVariable", "NextParameter", "EditorIndentSelection",
                                  "EmacsStyleIndent", "NextTemplateParameter", "ExpandLiveTemplateByTab", "BraceOrQuoteOut", "SplitChooser.NextWindow"},
    { "alt MINUS",                "RInsertAssignmentAction", "SECOND_STROKE_SHORTCUT"},
    { "alt SLASH",                "CodeCompletion", "HippieCompletion"},
    { "control D",                "$Delete", "Diff.ShowDiff", "CompareTwoFiles", "SendEOF", "FileChooser.GotoDesktop", "org.jetbrains.r.console.RConsoleView.REofAction"},
    { "control K",                "EditorCutLineEnd", "CheckinProject"},
    { "control M",                "EditorEnter", "EditorChooseLookupItem", "NextTemplateVariable", "Console.Execute"},
    { "control N",                "EditorDown", "FileChooser.NewFolder"},
    { "control P",                "EditorUp", "FileChooser.TogglePathShowing"},
    { "control R",                "org.jetbrains.plugins.ruby.rails.console.ReloadSources", "FindPrevious"},
    { "control SLASH",            "$Undo", "Graph.ActualSize"},
    { "control SPACE",            "EditorToggleStickySelection", "ChangesView.SetDefault"},
    { "control X,N",              "Diff.NextChange", "NextTab"},
    { "control X,P",              "Diff.PrevChange", "PreviousTab"},
    { "control UP",               "EditorBackwardParagraph", "EditorLookupUp", "JupyterSelectCellAboveAction", "MethodOverloadSwitchUp"},
    { "control DOWN",             "EditorForwardParagraph", "EditorLookupDown", "JupyterSelectCellBelowAction", "MethodOverloadSwitchDown"},
    { "ctrl V",                   "EditorPageDown", "Terminal.Paste"},
    { "control alt A",            "MethodUp", "ChangesView.AddUnversioned", "Diagram.DeselectAll"},
    { "control alt E",            "Console.History.Browse", "MethodDown", "PerforceDirect.Edit", "ToggleFindInSelection"},
    { "control alt G",            "GotoDeclaration", "org.jetbrains.plugins.ruby.rails.actions.generators.GeneratorsPopupAction", "Mvc.RunTarget"},
    { "control alt S",            "ShowSettings", "Find"},
    { "shift alt S",              "FindUsages", "context.save"},
    { "shift alt G",              "GotoChangedFile", "GotoClass", "hg4idea.QGotoFromPatches"},
    { "shift alt P",              "ParameterInfo", "hg4idea.QPushAction"},
    { "shift control X",          SECOND_STROKE, "com.jetbrains.php.framework.FrameworkRunConsoleAction"},
    { "shift ctrl DOWN",          "EditorForwardParagraphWithSelection", "ResizeToolWindowDown"},
    { "shift ctrl UP",            "EditorBackwardParagraphWithSelection", "ResizeToolWindowUp"},
    { "ESCAPE",                   SECOND_STROKE, "CloseGotItTooltip", "FocusEditor", "Terminal.SwitchFocusToEditor"},
    { "ctrl L",                   "EditorScrollToCenter", "Terminal.ClearBuffer"},
    });
    put("Visual Studio", new String[][] {
    { "F7",                       "CompileDirty", "NextDiff"},
    { "ctrl P",                   "FileChooser.TogglePathShowing", "Print"},
    { "ctrl F10",                 "RunToCursor", "UpdateRunningApplication"},
    { "F5",                       "Graph.RouteEdges", "Resume", "PoBrowserRefresh"},
    { "ctrl F1",                  "ExternalJavaDoc", "ShowErrorDescription"},
    { "shift alt CLOSE_BRACKET",  "EditorCodeBlockEnd", "EmmetNextEditPoint"},
    { "shift alt OPEN_BRACKET",   "EditorCodeBlockStart", "EmmetPreviousEditPoint"},
    { "ctrl T",                   "GotoChangedFile", "SearchEverywhere"},
    { "shift alt F11",            "FindUsagesInFile", "HighlightUsagesInFile"},
    { "ctrl alt B",               "ViewBreakpoints", "org.jetbrains.r.rendering.chunk.RunChunksAboveAction"},
    { "ctrl alt F",               "ActivateStructureToolWindow", "ShowFilterPopup"},
    { "shift ctrl DOWN",          "FindNext", "ResizeToolWindowDown"},
    { "ctrl R",                   SECOND_STROKE, "org.jetbrains.plugins.ruby.rails.console.ReloadSources"},
    { "shift ctrl UP",            "FindPrevious", "ResizeToolWindowUp"},
    { "ctrl SUBTRACT",            "Back", "CollapseAll", "CollapseExpandableComponent"},
    { "ctrl R,R",                 "ChangesView.Rename", "Git.Rename.Local.Branch", "Git.Reword.Commit", "RenameElement", "ShelvedChanges.Rename"},
    { "shift alt LEFT",           "Diff.ApplyRightSide", "EditorLeftWithSelection"},
    { "shift ctrl alt K",         "Git.Commit.And.Push.Executor", "Hg.Commit.And.Push.Executor"},
    { "ctrl E",                   SECOND_STROKE, "SwitcherIterateItems", "SwitcherRecentEditedChangedToggleCheckBox"},
    { "shift alt RIGHT",          "Diff.ApplyLeftSide", "EditorRightWithSelection"},
    { "ctrl N",                   "FileChooser.NewFolder", "NewElement"},
    { "ctrl F5",                  "Run", "RunClass"},
    { "shift F5",                 "Graph.ApplyCurrentLayout", "Stop"},
    { "ctrl alt W",               "ActivateWebToolWindow", "Vcs.UpdateProject"},
    { "shift alt ENTER",          "JupyterDebugAction", "ToggleFullScreen"},
    { "ctrl ENTER",               "Console.Execute.Multiline", "DirDiffMenu.SynchronizeDiff.All", "EditorStartNewLineBefore", "NotebookRunCellAction", "PyExecuteCellAction", "ViewSource", "org.jetbrains.r.actions.RunSelection", "Terminal.SmartCommandExecution.Run", "SplitChooser.Duplicate"},
    { "ctrl MINUS",               "Back", "CollapseAll", "CollapseExpandableComponent"},
    { "ctrl alt ENTER",           "ReformatCode", "org.jetbrains.r.actions.DebugSelection"},
    { "shift ctrl H",             "ChangesView.ShelveSilently", "ReplaceInPath"},
    { "shift ctrl alt P",         "IntroduceFunctionalParameter", "ReformatWithPrettierAction", "VcsShowPrevChangeMarker"},
    { "ctrl L",                   "$Cut", "Terminal.ClearBuffer"},
    { "alt O",                    "SwitchHeaderSource", "ExportToTextFile"},
    { "ctrl H",                   "Replace", "Vcs.ShowMessageHistory"},
    });
    put("Default for XWin", new String[][] {
    { "shift ctrl alt button1",   "EditorAddRectangularSelectionOnMouseDrag", "QuickEvaluateExpression"},
    { "shift ctrl C",             "CopyPaths", "DatabaseView.CopyDdlAction", "Terminal.CopySelectedText"},
    { "shift ctrl V",             "PasteMultiple", "Terminal.Paste"},
    });
    put("Default for GNOME", new String[][] {
    { "shift alt 1",              "SelectIn", "ProjectViewChangeView"},
    { "shift alt 7",              "IDEtalk.SearchUserHistory", "FindUsages"},
    { "shift alt LEFT",           "PreviousEditorTab", "Back", "Diff.ApplyRightSide"},
    { "shift alt RIGHT",          "NextEditorTab", "Forward", "Diff.ApplyLeftSide"},
    });
    put("Default for KDE", new String[][] {
    { "control 1",                "FileChooser.GotoHome", "ShowErrorDescription", "DuplicatesForm.SendToLeft"},
    { "control 2",                "FileChooser.GotoProject", "Stop", "DuplicatesForm.SendToRight"},
    { "control 3",                "FindWordAtCaret", "FileChooser.GotoModule"},
    { "control 5",                "Refresh", "Rerun"},
    { "shift alt 1",              "SelectIn", "ProjectViewChangeView"},
    { "shift alt 7",              "IDEtalk.SearchUserHistory", "FindUsages"},
    { "shift alt L",              "ReformatCode", "context.load"},
    });
    put("Eclipse", new String[][] {
    { "F2",                       "Console.TableResult.EditValue", "QuickJavaDoc", "XDebugger.SetValue"},
    { "F3",                       "GotoDeclaration", "PoToggleSelection"},
    { "F5",                       "Graph.RouteEdges", "StepInto", "PoBrowserRefresh"},
    { "alt DOWN",                 "ShowContent", "MoveStatementDown", "ShowSearchHistory"},
    { "alt HOME",                 "ViewNavigationBar", "ShowNavBar"},
    { "ctrl 1",                   "DuplicatesForm.SendToLeft", "FileChooser.GotoHome", "ShowIntentionActions"},
    { "control F10",              "ShowPopupMenu", "UpdateRunningApplication"},
    { "control D",                "EditorDeleteLine", "Diff.ShowDiff", "CompareTwoFiles", "SendEOF", "FileChooser.GotoDesktop", "org.jetbrains.r.console.RConsoleView.REofAction"},
    { "control N",                "ShowPopupMenu", "FileChooser.NewFolder"},
    { "control P",                "FileChooser.TogglePathShowing", "Print"},
    { "control R",                "RunToCursor", "org.jetbrains.plugins.ruby.rails.console.ReloadSources"},
    { "control U",                "EvaluateExpression", "CommanderSwapPanels"},
    { "control PAGE_DOWN",        "Diff.NextChange", "NextTab"},
    { "control PAGE_UP",          "Diff.PrevChange", "PreviousTab"},
    { "control F6",               "Diff.NextChange", "NextTab"},
    { "control alt DOWN",         "Console.TableResult.NextPage", "EditorDuplicateLines"},
    { "control alt E",            "Console.History.Browse", "ExecuteInPyConsoleAction", "PerforceDirect.Edit", "ToggleFindInSelection"},
    { "control alt LEFT",         "Diff.NextChange", "NextTab"},
    { "control alt RIGHT",        "Diff.PrevChange", "PreviousTab"},
    { "shift alt D",              "hg4idea.QFold", "Debug"},
    { "shift alt G",              "RerunTests", "hg4idea.QGotoFromPatches"},
    { "shift alt L",              "IntroduceVariable", "context.load"},
    { "shift alt P",              "hg4idea.QPushAction", "ImplementMethods"},
    { "shift alt R",              "RenameElement", "Git.Reword.Commit", "Git.Rename.Local.Branch", "ShelvedChanges.Rename", "ChangesView.Rename"},
    { "shift alt S",              "ShowPopupMenu", "context.save"},
    { "shift alt T",              "ShowPopupMenu", "tasks.switch"},
    { "shift control DOWN",       "ResizeToolWindowDown", "MethodDown"},
    { "shift control E",          "RecentLocations", "EditSource"},
    { "shift control F6",         "ChangeTypeSignature", "Diff.PrevChange", "PreviousTab"},
    { "shift control G",          "FindUsagesInFile", "ClassTemplateNavigation", "GoToClass"},
    { "shift control I",          "QuickImplementations", "XDebugger.Inspect"},
    { "shift control UP",         "ResizeToolWindowUp", "MethodUp"},
    { "shift control K",          "Vcs.Push", "FindPrevious"},
    { "shift control X",          "EditorToggleCase", "com.jetbrains.php.framework.FrameworkRunConsoleAction"},
    { "shift control T",          "GotoClass", "GotoChangedFile"},
    { "shift ctrl alt RIGHT",     "PreviousEditorTab", "Terminal.MoveToolWindowTabRight"},
    { "shift ctrl alt LEFT",      "NextEditorTab", "Terminal.MoveToolWindowTabLeft"},
    { "ctrl L",                   "GotoLine", "Terminal.ClearBuffer"},
    });
    put("NetBeans 6.5", new String[][] {
    { "F4",                       "RunToCursor", "EditSource"},
    { "F5",                       "Debugger.ResumeThread", "Resume", "Graph.RouteEdges", "PoBrowserRefresh"},
    { "alt DOWN",                 "GotoNextElementUnderCaretUsage", "ShowContent", "ShowSearchHistory"},
    { "control 1",                "ActivateProjectToolWindow", "DuplicatesForm.SendToLeft"},
    { "control 2",                "ActivateProjectToolWindow", "FileChooser.GotoProject", "DuplicatesForm.SendToRight"},
    { "control 3",                "ActivateProjectToolWindow", "FileChooser.GotoModule"},
    { "control BACK_SPACE",       "EditorDeleteToWordStart", "ToggleDockMode"},
    { "control DIVIDE",           "CollapseRegionRecursively", "Graph.ActualSize"},
    { "control M",                "Vcs.ShowMessageHistory", "Move"},
    { "control N",                "NewElement", "FileChooser.NewFolder"},
    { "control R",                "RenameElement", "org.jetbrains.plugins.ruby.rails.console.ReloadSources", "Git.Reword.Commit", "Git.Rename.Local.Branch", "ShelvedChanges.Rename", "ChangesView.Rename"},
    { "control U",                SECOND_STROKE, "CommanderSwapPanels"},
    { "control O",                "GotoClass", "GotoChangedFile"},
    { "control PERIOD",           "GotoNextError", "EditorChooseLookupItemDot"},
    { "control PAGE_DOWN",        "Diff.NextChange", "NextTab"},
    { "control PAGE_UP",          "Diff.PrevChange", "PreviousTab"},
    { "control alt DOWN",         "MethodDown", "NextOccurence", "Console.TableResult.NextPage"},
    { "control alt UP",           "MethodUp", "PreviousOccurence", "Console.TableResult.PreviousPage"},
    { "control alt shift P",      "Print", "ReformatWithPrettierAction"},
    { "shift F4",                 "RecentFiles", "SwitcherIterateItems", "SwitcherRecentEditedChangedToggleCheckBox", "Debugger.EditTypeSource", "Vcs.ShowMessageHistory", "EditSourceInNewWindow"},
    { "shift alt F9",             "ChooseDebugConfiguration", "ValidateXml", "ValidateJsp"},
    { "shift alt D",              "ToggleFloatingMode", "hg4idea.QFold"},
    { "shift control DOWN",       "EditorDuplicate", "ResizeToolWindowDown", },
    { "shift control F7",         "HighlightUsagesInFile", "XDebugger.NewWatch"},
    { "shift control UP",         "EditorDuplicate", "ResizeToolWindowUp", },
    { "shift ctrl C",             "CommentByLineComment", "DatabaseView.CopyDdlAction"},
    { "shift control H",          "ChangesView.ShelveSilently", "ReplaceInPath"},
    { "shift control K",          "HippieCompletion", "Vcs.Push"},
    { "shift control M",          "RInsertPipeAction", "ShowBookmarks"},
    { "control alt E",            "Console.History.Browse", "ExecuteInPyConsoleAction", "PerforceDirect.Edit", "ToggleFindInSelection"},
    { "shift ctrl BACK_SLASH",    "GotoUrlAction", "SmartTypeCompletion"},
    });
    put("Eclipse (Mac OS X)", new String[][] {
    { "meta BACK_SPACE",          "EditorDeleteToWordStart", "$Delete"},
    { "F2",                       "Console.TableResult.EditValue", "QuickJavaDoc", "XDebugger.SetValue", "Git.Reword.Commit", "Git.Rename.Local.Branch", "ShelvedChanges.Rename", "ChangesView.Rename"},
    { "F3",                       "GotoDeclaration", "EditSource", "PoToggleSelection"},
    { "F5",                       "StepInto", "Graph.RouteEdges", "PoBrowserRefresh"},
    { "alt DOWN",                 "MoveStatementDown", "MethodOverloadSwitchDown", "ShowSearchHistory"},
    { "alt UP",                   "MoveStatementUp", "MethodOverloadSwitchUp"},
    { "control PERIOD",           "EditorChooseLookupItemDot", "HippieCompletion"},
    { "meta 1",                   "FileChooser.GotoHome", "ShowIntentionActions", "DuplicatesForm.SendToLeft"},
    { "meta 3",                   "FileChooser.GotoModule", "GotoAction"},
    { "meta D",                   "EditorDeleteLine", "Diff.ShowDiff", "CompareTwoFiles", "SendEOF", "FileChooser.GotoDesktop"},
    { "meta I",                   "DatabaseView.PropertiesAction", "AutoIndentLines"},
    { "meta P",                   "FileChooser.TogglePathShowing", "Print"},
    { "meta R",                   "org.jetbrains.plugins.ruby.rails.console.ReloadSources", "RunToCursor"},
    { "meta U",                   "CommanderSwapPanels", "EvaluateExpression"},
    { "meta W",                   "CloseContent", "CloseActiveTab"},
    { "meta F6",                  "Diff.NextChange", "NextTab"},
    { "shift meta T",             "GotoClass", "GotoChangedFile"},
    { "meta alt B",               "ShowNavBar", "org.jetbrains.r.rendering.chunk.RunChunksAboveAction"},
    { "meta alt C",               "ChangeSignature", "org.jetbrains.r.rendering.chunk.RunChunkAction"},
    { "meta alt I",               "Inline", "RMarkdownNewChunk"},
    { "meta alt LEFT",            "Diff.PrevChange", "PreviousTab"},
    { "meta alt RIGHT",           "Diff.NextChange", "NextTab"},
    { "meta alt DOWN",            "Console.TableResult.NextPage", "EditorDuplicateLines"},
    { "shift meta G",             "ClassTemplateNavigation", "GoToClass", "FindUsages"},
    { "shift meta K",             "Vcs.Push", "FindPrevious"},
    { "shift meta X",             "EditorToggleCase", "com.jetbrains.php.framework.FrameworkRunConsoleAction"},
    { "shift meta U",             "FindUsagesInFile", "ShelveChanges.UnshelveWithDialog"},
    { "shift meta F6",            "Diff.PrevChange", "PreviousTab"},
    { "control shift alt Z",      "Vcs.RollbackChangedLines", "ChangesView.Revert"},
    { "meta K",                   "FindNext", "Terminal.ClearBuffer"},
    { "shift ctrl DOWN",          "EditorUnSelectWord", "ShowContent"},
    });
    put("Sublime Text", new String[][] {
    { "F2",                       "ChangesView.Rename", "Console.TableResult.EditValue", "Git.Reword.Commit", "Git.Rename.Local.Branch", "GotoNextBookmark", "GuiDesigner.EditComponent", "GuiDesigner.EditGroup", "ShelvedChanges.Rename", "XDebugger.SetValue"},
    { "alt MINUS",                "Back", "RInsertAssignmentAction"},
    { "ctrl ADD",                 "EditorIncreaseFontSize", "ExpandAll", "ExpandExpandableComponent"},
    { "ctrl D",                   "CompareTwoFiles", "Diff.ShowDiff", "FileChooser.GotoDesktop", "SelectNextOccurrence", "SendEOF", "org.jetbrains.r.console.RConsoleView.REofAction"},
    { "ctrl ENTER",               "Console.Execute.Multiline", "DirDiffMenu.SynchronizeDiff.All", "EditorStartNewLine", "NotebookRunCellAction", "PyExecuteCellAction", "ViewSource", "org.jetbrains.r.actions.RunSelection", "Terminal.SmartCommandExecution.Run", "SplitChooser.Duplicate"},
    { "ctrl EQUALS",              "EditorIncreaseFontSize", "ExpandAll", "ExpandExpandableComponent"},
    { "ctrl M",                   "EditorMatchBrace", "Vcs.ShowMessageHistory"},
    { "ctrl MINUS",               "CollapseAll", "CollapseExpandableComponent", "EditorDecreaseFontSize"},
    { "ctrl N",                   "FileChooser.NewFolder", "NewElement"},
    { "ctrl P",                   "FileChooser.TogglePathShowing", "GotoFile"},
    { "ctrl R",                   "FileStructurePopup", "org.jetbrains.plugins.ruby.rails.console.ReloadSources"},
    { "ctrl SUBTRACT",            "CollapseAll", "CollapseExpandableComponent", "EditorDecreaseFontSize"},
    { "ctrl V",                   "EditorPasteSimple", "Terminal.Paste"},
    { "ctrl alt DOWN",            "Console.TableResult.NextPage", "EditorCloneCaretBelow"},
    { "ctrl alt UP",              "Console.TableResult.PreviousPage", "EditorCloneCaretAbove"},
    { "shift ENTER",              "CollapseExpandableComponent", "Console.TableResult.EditValueMaximized", "DatabaseView.PropertiesAction", "EditorSplitLine", "ExpandExpandableComponent", "NotebookRunCellSelectBelowAction", "OpenInRightSplit", "RConsoleNextLineAction"},
    { "shift INSERT",             "EditorPasteSimple", "Terminal.Paste"},
    { "shift ctrl D",             "Console.TableResult.CompareCells", "EditorDuplicate", "Uml.ShowDiff"},
    { "shift ctrl M",             "EditorSelectWord", "RInsertPipeAction"},
    { "shift ctrl DOWN",          "MoveLineDown", "ResizeToolWindowDown"},
    { "shift ctrl ENTER",         "Console.Jpa.GenerateSql", "EditorStartNewLineBefore", "org.jetbrains.r.actions.RRunAction", "Terminal.SmartCommandExecution.Debug"},
    { "shift ctrl T",             "ReopenClosedTab", "ServiceView.ShowServices"},
    { "shift ctrl UP",            "MoveLineUp", "ResizeToolWindowUp"},
    { "ctrl L",                   "EditorSelectLine", "Terminal.ClearBuffer"},
    });
    put("Sublime Text (Mac OS X)", new String[][] {
    { "F2",                       "ChangesView.Rename", "Console.TableResult.EditValue", "Git.Reword.Commit", "Git.Rename.Local.Branch", "GotoNextBookmark", "GuiDesigner.EditComponent", "GuiDesigner.EditGroup", "ShelvedChanges.Rename", "XDebugger.SetValue"},
    { "meta ADD",                 "EditorIncreaseFontSize", "ExpandAll", "ExpandExpandableComponent"},
    { "meta BACK_SPACE",          "EditorDeleteToLineStart", "$Delete"},
    { "meta D",                   "CompareTwoFiles", "Diff.ShowDiff", "FileChooser.GotoDesktop", "SelectNextOccurrence", "SendEOF"},
    { "meta ENTER",               "Console.Execute.Multiline", "DirDiffMenu.SynchronizeDiff.All", "EditorStartNewLine", "ViewSource", "org.jetbrains.r.actions.RunSelection", "Terminal.SmartCommandExecution.Run", "SplitChooser.Duplicate"},
    { "meta EQUALS",              "EditorIncreaseFontSize", "ExpandAll", "ExpandExpandableComponent"},
    { "meta I",                   "DatabaseView.PropertiesAction", "IncrementalSearch"},
    { "meta MINUS",               "CollapseAll", "CollapseExpandableComponent", "EditorDecreaseFontSize"},
    { "meta P",                   "FileChooser.TogglePathShowing", "GotoFile"},
    { "meta R",                   "FileStructurePopup", "Refresh", "Rerun", "org.jetbrains.plugins.ruby.rails.console.ReloadSources"},
    { "meta SUBTRACT",            "CollapseAll", "CollapseExpandableComponent", "EditorDecreaseFontSize"},
    { "meta V",                   "EditorPasteSimple", "Terminal.Paste"},
    { "meta alt DOWN",            "Console.TableResult.NextPage", "GotoDeclaration"},
    { "meta alt G",               "DatabaseView.SqlGenerator", "FindWordAtCaret", "org.jetbrains.plugins.ruby.rails.actions.generators.GeneratorsPopupAction"},
    { "shift ENTER",              "CollapseExpandableComponent", "Console.TableResult.EditValueMaximized", "EditorSplitLine", "ExpandExpandableComponent", "NotebookRunCellSelectBelowAction", "OpenInRightSplit", "RConsoleNextLineAction"},
    { "shift meta D",             "Console.TableResult.CompareCells", "EditorDuplicate", "Uml.ShowDiff"},
    { "shift meta ENTER",         "Console.Jpa.GenerateSql", "EditorStartNewLineBefore",  "org.jetbrains.r.actions.RRunAction", "Terminal.SmartCommandExecution.Debug"},
    { "shift meta T",             "ReopenClosedTab", "ServiceView.ShowServices"},
    { "meta L",                   "EditorSelectLine", "Terminal.ClearBuffer"},
    { "meta K",                   "SECOND_STROKE_SHORTCUT", "Terminal.ClearBuffer"},
    { "shift ctrl DOWN",          "EditorCloneCaretBelow", "ShowContent"},
    });
    put("macOS System Shortcuts", new String[][] {
      { "ctrl C",                   "Vcs.QuickListPopupAction", "org.jetbrains.r.console.RConsoleView.RInterruptAction"},
      { "meta I",                   "DatabaseView.PropertiesAction", "QuickJavaDoc"},
      { "shift meta T",             "ReopenClosedTab", "ServiceView.ShowServices"},
      { "ctrl O",                   "EditorSplitLine", "ExportToTextFile"},
      { "meta DOWN",                "EditSourceNotInEditor", "EditorTextEnd"},
      { "meta alt I",               "Move", "RMarkdownNewChunk"},
      { "shift meta DOWN",          "EditorTextEndWithSelection", "ResizeToolWindowDown"},
      { "shift meta UP",            "EditorTextStartWithSelection", "ResizeToolWindowUp"},
      { "ctrl ENTER",               "EditorSplitLine", "NotebookRunCellAction", "PyExecuteCellAction"},
      { "meta CLOSE_BRACKET",       "Forward", "NextDiff"},
      { "meta OPEN_BRACKET",        "Back", "PreviousDiff"},
      { "meta alt E",               "Console.History.Browse", "GotoNextElementUnderCaretUsage", "PerforceDirect.Edit"},
      { "ctrl meta M",              "ChangesView.Move", "Vcs.MoveChangedLinesToChangelist"},
      { "meta alt DOWN",            "Console.TableResult.NextPage", "JupyterSelectCellBelowAction", "NextOccurence"},
      { "meta alt UP",              "Console.TableResult.PreviousPage", "JupyterSelectCellAboveAction", "PreviousOccurence"},
    });
  }};
  // @formatter:on

  @NonNls protected static final Set<String> DEFAULT_UNKNOWN_ACTION_IDS = Set.of(
    "ActivateCommitToolWindow", "ActivateVersionControlToolWindow", "ActivateFavoritesToolWindow", "ActivateCommanderToolWindow",
    "ActivateServicesToolWindow",
    "ActivateMessagesToolWindow", "ActivateProjectToolWindow", "ActivateStructureToolWindow",
    "ActivateProblemsViewToolWindow",
    "ActivateTODOToolWindow", "ActivateWebToolWindow", "ActivatePaletteToolWindow", "ActivateTerminalToolWindow",
    "IDEtalk.SearchUserHistory", "IDEtalk.Rename", "CIDR.Lang.Cpp.GenerateDefinitions",
    "ActivateErrorsInSolutionToolWindow", "ActivateUnitTestsToolWindow",
    "ActivateDatabaseToolWindow", "ActivateBuildToolWindow", "ActivateNuGetToolWindow", "RiderBackendAction-EncapsulateField",
    "SwitchHeaderSource", "BuildProject", "RebuildProject", "BuildSolutionAction", "RebuildSolutionAction",
    "ActivateInspectionResultsToolWindow",
    ""
  );

  @NonNls protected static final Set<String> DEFAULT_BOUND_ACTIONS = Set.of(
    "EditorDelete",
    "List-selectLastRowExtendSelection",
    "List-selectLastRow",
    "List-selectFirstRowExtendSelection",
    "List-selectFirstRow",
    "Tree-selectLastExtendSelection",
    "Tree-selectLast",
    "Tree-selectFirstExtendSelection",
    "Tree-selectFirst"
  );
  private static final List<String> DEFAULT_CONFLICT_SAFE_GROUPS =
    List.of("Log.KeymapGroup", "UsageGroupingActionGroup", "SearchEverywhereActions", "Images.ThumbnailViewActions", "Images.ImageViewActions",
            "ArrangementRulesGroup", "ServiceViewTreeToolbar", "TodoViewGroupByGroup", "ChangesView.GroupBy");

  @Override
  protected Collection<String> getConflictSafeGroups() {
    return DEFAULT_CONFLICT_SAFE_GROUPS;
  }

  @Override
  protected String getGroupForUnknownAction(@NotNull String actionId) {
    if (actionId.equals("ChangesView.GroupBy.Module")) {
      return "ChangesView.GroupBy";
    }
    return null;
  }
}
